@@ -4,10 +4,12 @@ answer:
 http://stackoverflow.com/questions/2225995/how-can-i-create-stacked-line-graph-with-matplotlib
 
 (http://stackoverflow.com/users/66549/doug)
-
 """
+
 import numpy as np
 
+from matplotlib import _api
+
 __all__ = ['stackplot']
 
 
@@ -19,15 +21,14 @@ def stackplot(axes, x, *args,
 
     Parameters
     ----------
-    x : 1d array of dimension N
-
-    y : 2d array (dimension MxN), or sequence of 1d arrays (each dimension 1xN)
+    x : (N,) array-like
 
+    y : (M, N) array-like
         The data is assumed to be unstacked. Each of the following
         calls is legal::
 
-            stackplot(x, y)               # where y is MxN
-            stackplot(x, y1, y2, y3, y4)  # where y1, y2, y3, y4, are all 1xNm
+            stackplot(x, y)           # where y has shape (M, N)
+            stackplot(x, y1, y2, y3)  # where y1, y2, y3, y4 have length N
 
     baseline : {'zero', 'sym', 'wiggle', 'weighted_wiggle'}
         Method used to calculate the baseline:
@@ -40,20 +41,19 @@ def stackplot(axes, x, *args,
           size of each layer. It is also called 'Streamgraph'-layout. More
           details can be found at http://leebyron.com/streamgraph/.
 
-    labels : Length N sequence of strings
+    labels : Length N list of str
         Labels to assign to each data series.
 
-    colors : Length N sequence of colors
+    colors : Length N list of color
         A list or tuple of colors. These will be cycled through and used to
         colour the stacked areas.
 
-    **kwargs :
-        All other keyword arguments are passed to `Axes.fill_between()`.
-
+    **kwargs
+        All other keyword arguments are passed to `.Axes.fill_between`.
 
     Returns
     -------
-    list : list of `.PolyCollection`
+    list of `.PolyCollection`
         A list of `.PolyCollection` instances, one for each element in the
         stacked area plot.
     """
@@ -68,6 +68,8 @@ def stackplot(axes, x, *args,
     # We'll need a float buffer for the upcoming calculations.
     stack = np.cumsum(y, axis=0, dtype=np.promote_types(y.dtype, np.float32))
 
+    _api.check_in_list(['zero', 'sym', 'wiggle', 'weighted_wiggle'],
+                       baseline=baseline)
     if baseline == 'zero':
         first_line = 0.
 
@@ -82,7 +84,6 @@ def stackplot(axes, x, *args,
         stack += first_line
 
     elif baseline == 'weighted_wiggle':
-        m, n = y.shape
         total = np.sum(y, 0)
         # multiply by 1/total (or zero) to avoid infinities in the division:
         inv_total = np.zeros_like(total)
@@ -98,11 +99,6 @@ def stackplot(axes, x, *args,
         first_line = center - 0.5 * total
         stack += first_line
 
-    else:
-        errstr = "Baseline method %s not recognised. " % baseline
-        errstr += "Expected 'zero', 'sym', 'wiggle' or 'weighted_wiggle'"
-        raise ValueError(errstr)
-
     # Color between x = 0 and the first array.
     color = axes._get_lines.get_next_color()
     coll = axes.fill_between(x, first_line, stack[0, :],
